/*
 * Copyright (c) 1999, 2000, 2001, 2002 WIDE Project
 * All rights reserved.
 *
 * Author : Akimichi OGAWA (akimichi@sfc.wide.ad.jp)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code MUST retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form MUST reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    MUST display the following acknowledgement:
 *      This product includes software developed by Akimichi OGAWA.
 * 4. The name of the author MAY NOT be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifdef HAVE_CONFIG_H
#include <dvts-config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <Carbon/Carbon.h>

#include <libdv/dv.h>

#define	USE_COLOR	0x00000001
#define USE_GREY	0x00000002

#define  WIDTH	720
#define  HEIGHT 480

static struct xy mb_pos[5][27][4];

static void _init_macro_block_position __P((void));

int
xdvshow_open_window(char *window_name)
{
  WindowRef window;
  const Rect defaultBounds = {100, 100, 100+WIDTH, 100+HEIGHT};

  _init_macro_block_position();

  CreateNewWindow(kDocumentWindowClass,
                  kWindowStandardHandlerAttribute,
                  &defaultBounds,
                  &window);
  SetWindowTitleWithCFString(window, CFSTR("dvshow"));

  ShowWindow(window);

  return(1);
}

int
xdvshow_close_window(void)
{
  return(1);
}

void
xdvshow_repaint(void)
{
}

int
xdvshow_render(struct rgb_data525_60 *rgb)
{
  int16_t r, g, b;
  int16_t x, y;
  int16_t *r_ptr, *g_ptr, *b_ptr;

  r_ptr = &rgb->r[0][0];
  g_ptr = &rgb->g[0][0];
  b_ptr = &rgb->b[0][0];

  for (x=0; x<WIDTH; x++) {
    for (y=0; y<HEIGHT; y++) {
      r = *r_ptr;
      if (r > 255) {
        r = 255;
      }
      if (r < 0) {
        r = 0;
      }
      r_ptr++;

      g = *g_ptr;
      if (g > 255) {
        g = 255;
      }
      if (g < 0) {
        g = 0;
      }
      g_ptr++;

      b = *b_ptr;
      if (b > 255) {
        b = 255;
      }
      if (b < 0) {
        b = 0;
      }
      b_ptr++;

/*      XPutPixel(ximage, x, y, color_table[r][g][b]); */
    }
  }

  xdvshow_repaint();

  return(1);
}

/***************************/

static void
_init_macro_block_position(void)
{
  int i, j, k, m;
  int count;

  for(m=0; m<3; m++) {
    count = 0;
    for(i=0; i<3; i++) {
      for(j=0; j<6; j++) {
        for(k=0; k<4; k++) {
          mb_pos[m*2][count][k].x = 64*i + k*8 + m*288;
          mb_pos[m*2][count][k].y = j*8;
        }
        count++;
        if(count >= 27) {
          break;
        }
      }
      for(j=5; j>=0; j--) {
        if(count >= 27) {
          break;
        }
        for(k=0; k<4; k++) {
          mb_pos[m*2][count][k].x = 32 + 64*i + k*8 + m*288;
          mb_pos[m*2][count][k].y = j*8;
        }
        count++;
      }
    }
  }

  for(m=0; m<2; m++) {
    count = 0;
    for(j=0; j<3; j++) {
      for(k=0; k<4; k++) {
        mb_pos[1+m*2][count][k].x = 128 + m*288 + k*8;
        mb_pos[1+m*2][count][k].y = 24  + j*8;
      }
      count++;
    }
  }

  for(m=0; m<2; m++) {
    count = 3;
    for(i=0; i<2; i++) {
      for(j=5; j>=0; j--) {
        for(k=0; k<4; k++) {
          mb_pos[1+m*2][count][k].x = 128 + 32 + 64*i + k*8 + m*288;
          mb_pos[1+m*2][count][k].y = j*8;
        }
        count++;
      }
      for(j=0; j<6; j++) {
        for(k=0; k<4; k++) {
          mb_pos[1+m*2][count][k].x = 128 + 64 + 64*i + k*8 + m*288;
          mb_pos[1+m*2][count][k].y = j*8;
        }
        count++;
      }
    }
  }

  for(i=24; i<27; i++) {
    for(k=0; k<2; k++) {
      mb_pos[4][i][k].x   = 720 - 16 + k*8;
      mb_pos[4][i][k].y   = (i - 24)*16;
      mb_pos[4][i][k+2].x = 720 - 16 + k*8;
      mb_pos[4][i][k+2].y = 8 + (i - 24)*16;
    }
  }
}
