/*
 * Copyright (c) 1999, 2000, 2001, 2002 WIDE Project
 * All rights reserved.
 *
 * Author : Akimichi OGAWA (akimichi@sfc.wide.ad.jp)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code MUST retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form MUST reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    MUST display the following acknowledgement:
 *      This product includes software developed by Akimichi OGAWA.
 * 4. The name of the author MAY NOT be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>

#ifdef	FREEBSD_4
#include <sys/soundcard.h>
#endif	/* FREEBSD_4 */

#ifdef	NETBSD
#include <sys/ioccom.h>
#include <soundcard.h>
#endif	/* NETBSD */

#ifdef	LINUX
#include <sys/ioctl.h>
#include <linux/soundcard.h>
#endif	/* LINUX */

#include <libdv/dv.h>

#include "xdvshow-audio.h"

int audio_fd;
int init_finished = 0;

static int _init_audio_device __P((dv_audio_t *));
int
xdvshow_init_audio(char *audio_dev, dv_audio_t *audio)
{
  int ret;

  if (audio_dev == NULL) {
    return(-1);
  }

  audio_fd = open(audio_dev, O_WRONLY);
  if (audio_fd < 0) {
    perror("open audio");
    return(-1);
  }

  ret = _init_audio_device(audio);
  if (ret < 0) {
    return(-1);
  }

  return(1);
}

int
xdvshow_play_audio(dv_audio_t *audio, int16_t **audio_buffers)
{
  int ch, i, j=0;
  static int16_t outbuf[DV_AUDIO_MAX_SAMPLES * 4];

  for (i=0; i<audio->samples_this_frame; i++) {
    for (ch=0; ch<audio->num_channels; ch++) {
      outbuf[j] = audio_buffers[ch][i];
      j++;
    }
  }

  write(audio_fd,
        outbuf,
        audio->samples_this_frame * audio->num_channels * sizeof(u_int16_t));

  return(1);
}

static int
_init_audio_device(dv_audio_t *audio)
{
  int ret;
  int format;
  int fragment_request;
  int channels;
  int rate;

  format = AFMT_S16_LE;
  fragment_request = 0x000300B;
  channels = audio->num_channels;
  rate = audio->frequency;

  ret = ioctl(audio_fd, SNDCTL_DSP_SETFRAGMENT, &fragment_request);
  if (ret < 0) {
    perror("SNDCTL_DSP_SETFRAGMENT\n");
    return(-1);
  }

  ret = ioctl(audio_fd, SOUND_PCM_WRITE_BITS, &format);
  if (ret < 0) {
    perror("SOUND_PCM_WRITE_BITS\n");
    return(-1);
  }

  ret = ioctl(audio_fd, SOUND_PCM_WRITE_CHANNELS, &channels);
  if (ret < 0) {
    perror("SOUND_PCM_WRITE_CHANNELS\n");
    return(-1);
  }

  ret = ioctl(audio_fd, SOUND_PCM_WRITE_RATE, &rate);
  if (ret < 0) {
    perror("SOUND_PCM_WRITE_RATE\n");
    fprintf(stderr, "rate : %d\n", rate);
    return(-1);
  }

  return(1);
}
