/*
 * Copyright (c) 1999, 2000, 2001, 2002 WIDE Project
 * All rights reserved.
 *
 * Author : Akimichi OGAWA (akimichi@sfc.wide.ad.jp)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code MUST retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form MUST reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    MUST display the following acknowledgement:
 *      This product includes software developed by Akimichi OGAWA.
 * 4. The name of the author MAY NOT be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <netinet/in.h>

#include <libdv/dv_types.h>

#include <xdvshow-file.h>

static FILE *_file = NULL;

extern u_int32_t dv_format_type;

int dseq_max;

int
xdvshow_prepare_file(void *data)
{
  char *filename = data;

  if (filename == NULL) {
    return(-1);
  }

  _file = fopen(filename, "r");
  if (_file == NULL) {
    return(-1);
  }

  return(1);
}

void
xdvshow_close_file()
{
  fclose(_file);
}

int
xdvshow_read_audio_file(u_char **audioframe)
{
  /* DO NOTHING FOR NOW */

  return(1);
}

int
xdvshow_read_file(u_char **frame_return)
{
  int n;

  static u_char dvframe[144000];

  u_int32_t dvdif[80 / 4];
  int sct, dseq, dbn;

  *frame_return = dvframe;

  switch (dv_format_type) {
    case e_dv_system_525_60:
      dseq_max = 9;
      break;

    case e_dv_system_625_50:
      dseq_max = 11;
      break;

    default:
      return(-1);
  }

  if (feof(_file)) {
    return(0);
  }

  while (1) {
    n = read(fileno(_file), (char *)&dvdif, 80);
    if (n < 1) {
      if (feof(_file)) {
        return(0);
      }
      perror("read");
      return(-1);
    }
                   
    dbn  = (ntohl(dvdif[0]) >> 8) & 0xff;
    dseq = (ntohl(dvdif[0]) >> 20) & 0xf;
    sct  = (ntohl(dvdif[0]) >> 29) & 0x7;

    switch (sct) {
      case 0x0: /* SCT_HEADER */
      case 0x1: /* SCT_SUBCODE */
        break;

      case 0x2: /* SCT_VAUX */
        memcpy(&dvframe[(dseq * 150 + dbn + 3) * 80], &dvdif, 80);
        break;

      case 0x3: /* SCT_AUDIO */
        memcpy(&dvframe[(dseq * 150 + 6 + dbn*16) * 80], &dvdif, 80);
        break;

      case 0x4: /* SCT_VIDEO */
        memcpy(&dvframe[(dseq * 150 + 7 + dbn + dbn/15) * 80], &dvdif, 80);
        if (dseq == dseq_max && dbn == 134) {
          return(1);
        }
        break;

      default:
        printf("INVALID FILE FORMAT!!!\n");
        return(-1);
    }
  }

  return(-1);
}
