/*
 * Copyright (c) 1999, 2000, 2001, 2002 WIDE Project
 * Author : Akimichi OGAWA (akimichi@sfc.wide.ad.jp)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code MUST retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form MUST reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    MUST display the following acknowledgement:
 *      This product includes software developed by Akimichi OGAWA.
 * 4. The name of the author MAY NOT be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <signal.h>
#include <sys/socket.h>

#include <pthread.h>

#ifdef	LINUX
#include <getopt.h>
#endif	/* LINUX */

#include <libdv/dv.h>

#include <xdvshow-info.h>
#ifdef	USE_AQUA
#include <xdvshow-aqua.h>
#else	/* USE_AQUA */
#ifdef	HAVE_SDL
#include <xdvshow-sdl.h>
#else	/* HAVE_SDL */
#include <xdvshow-x11.h>
#endif	/* HAVE_SDL */
#endif /* USE_AQUA */
#include <xdvshow-file.h>
#include <xdvshow-shm.h>
#include <xdvshow-audio.h>
#include <xdvshow-flags.h>

#define		XDVSHOW_PROGNAME	"xdvshow"

u_int32_t	flags = 0x0;
u_int32_t	dv_format_type = e_dv_system_525_60;
int (*input_func) __P((u_char **));
int (*input_unlock_func) __P(());

char *prog_name = XDVSHOW_PROGNAME;

void *video_thread_func __P((void *));
u_char *videodata;

int
main(int argc, char *argv[])
{
  int ret;

  u_char *dvframe;
  u_char *audioframe;

  int16_t *audio_buffers[4];
  int i;

  char *audio_dev = NULL;
  dv_decoder_t *dv_dec;

  pthread_t thread;

  extern char *optarg;
  int op;

  void *open_option = NULL;

  int (*input_open) __P((void *));

  int (*audio_input_func) __P((u_char **));
  int (*audio_unlock_func) __P(());

  void (*input_close) __P((void));
  void (*input_signal) __P((int));

  struct xdvshow_rtp_param rtp_param;

  memset(&rtp_param, 0, sizeof(rtp_param));

  dv_dec = dv_decoder_new(FALSE, FALSE, FALSE);
  if (dv_dec == NULL) {
    fprintf(stderr, "ERROR : dv_decoder_new\n");
    return(1);
  }

  dv_dec->quality = DV_QUALITY_BEST;

  input_open  = xdvshow_prepare_shm;
  input_func  = xdvshow_read_shm;
  input_unlock_func = xdvshow_read_unlock;
  audio_input_func  = xdvshow_read_audio_shm;
  audio_unlock_func = xdvshow_read_audio_unlock;
  input_close = xdvshow_close_shm;
  input_signal = xdvshow_signal_shm;
  open_option = &rtp_param;

#ifdef	ENABLE_INET6
  rtp_param.s_addr.ss_family = AF_INET6;
#else	/* ENABLE_INET6 */
  rtp_param.s_addr.ss_family = AF_INET;
#endif	/* ENABLE_INET6 */

  rtp_param.port = 8000;

  while ((op = getopt(argc, argv, "vF:tH46P:j:M:A:N2Q:p")) > 0) {
    switch (op) {
      case 'v':
        show_version();
        exit(0);
        break;

      case 'F':
        input_open = xdvshow_prepare_file;
        input_func = xdvshow_read_file;
        audio_input_func = xdvshow_read_audio_file;
        input_close = xdvshow_close_file;
        input_signal = NULL;
        open_option = optarg;
        break;

      case 't':
        flags |= SHOW_TIME;
        break;

      case '4':
        rtp_param.s_addr.ss_family = AF_INET;
        break;

#ifdef	ENABLE_INET6
      case '6':
        rtp_param.s_addr.ss_family = AF_INET6;
        break;
#endif	/* ENABLE_INET6 */

      case 'P':
        rtp_param.port = atoi(optarg);
        if (rtp_param.port < 1) {
          printf("Invalid port number : %d\n", rtp_param.port);
          return(1);
        }
        break;

      case 'j':
        rtp_param.multi_addr_str = optarg;
        break;

      case 'M':
        rtp_param.multi_ifname = optarg;
        break;

      case 'A':
        flags |= USE_AUDIO;
        audio_dev = optarg;
        break;

      case 'N':
        flags |= DO_NOT_DECODE_VIDEO;
        break;

      case '2':
        flags |= HALF_SIZE;
        break;

      case 'Q':
        {
          int quality;
          quality = atoi(optarg);
          switch (quality) {
            case 1:
              dv_dec->quality = DV_QUALITY_FASTEST;
              break;
            case 2:
              dv_dec->quality = DV_QUALITY_AC_1;
              break;
            case 3:
              dv_dec->quality = DV_QUALITY_COLOR | DV_QUALITY_DC;
              break;
            case 4:
              dv_dec->quality = DV_QUALITY_COLOR | DV_QUALITY_AC_1;
              break;
            case 5:
              break;
            default:
              break;
          }
        }
        break;

      case 'p':
        dv_format_type = e_dv_system_625_50;
        break;

      case 'H':
      default:
        show_usage(argv[0]);
        return(1);
        break;
    }
  }

  if (flags & DO_NOT_DECODE_VIDEO && !(flags & USE_AUDIO)) {
    fprintf(stderr, "You requested NO VIDEO and NO AUDIO\n");
    fprintf(stderr, "What should I do ?\n");
    return(1);
  }

  if (input_signal != NULL) {
    signal(SIGINT, input_signal);
  }

  if (input_open == NULL || input_func == NULL || input_close == NULL) {
    printf("No modules specified\n");
    return(1);
  }

  ret = (*input_open)(open_option);
  if (ret < 0) {
    printf("open failed\n");
    kill(getpid(), SIGINT);
    return(1);
  }

  ret = (*input_func)(&dvframe);
  if (ret < 0) {
    printf("read dv failed\n");
    kill(getpid(), SIGINT);
    return(1);
  }
  dv_parse_header(dv_dec, dvframe);
  dv_format_type = dv_dec->system;

  if (input_unlock_func) {
    (*input_unlock_func)();
  }

  if (flags & USE_AUDIO) {
    ret = xdvshow_init_audio(audio_dev, dv_dec->audio);
    if (ret < 0) {
      kill(getpid(), SIGINT);
      return(1);
    }

    for (i=0; i<4; i++) {
      audio_buffers[i] = malloc(DV_AUDIO_MAX_SAMPLES*sizeof(u_int16_t));

      if (audio_buffers[i] == NULL) {
        printf("failed to init audio\n");
        return(1);
      }

      memset(audio_buffers[i], 0, DV_AUDIO_MAX_SAMPLES*sizeof(u_int16_t));
    }
  }

  if (!(flags & DO_NOT_DECODE_VIDEO)) {
    memset(&thread, 0, sizeof(thread));
    ret = pthread_create(&thread, NULL, video_thread_func, (void *)dv_dec);
    if (ret != 0) { 
      fprintf(stderr, "pthread create : %d\n", ret);
      return(1);
    }
  }

  if (flags & USE_AUDIO) {
    while (1) {
      (*audio_input_func)(&audioframe);

      dv_decode_full_audio(dv_dec, audioframe, audio_buffers);
      ret = xdvshow_play_audio(dv_dec->audio, audio_buffers);
      if (ret < 0) {
        break;
      }

      if (audio_unlock_func) {
        (*audio_unlock_func)();
      }
    }
  } else {
    while (1) {
      sleep(30);
    }
  }

  xdvshow_close_window();

  dv_decoder_free(dv_dec);

  kill(getpid(), SIGINT);

  return(0);
}

void *
video_thread_func(void *data)
{
  int ret;

  dv_decoder_t *dv_dec = NULL;

  struct x_params param;

  struct timeval tv, tv_prev;

  memset(&param, 0, sizeof(param));

  dv_dec = (dv_decoder_t *)data;

  memset(&tv, 0, sizeof(tv));
  gettimeofday(&tv, NULL);
  memcpy(&tv_prev, &tv, sizeof(tv));

  param.dv_format_type = dv_format_type;

  ret = xdvshow_open_window(prog_name, &param);
  if (ret < 0) {
    printf("failed to open window\n");
    kill(getpid(), SIGINT);
    return(NULL);
  }

  if (flags & SHOW_TIME) {
    while (1) {
      ret = (*input_func)(&videodata);
      if (ret == 0) {
        kill(getpid(), SIGINT);
        return(NULL);
      } else if (ret < 0) {
        printf("read dv failed\n");
        kill(getpid(), SIGINT);
        return(NULL);
      }
      dv_decode_full_frame(dv_dec,
                           videodata,
                           param.decode_format,
                           param.pixels,
                           param.pitches);

      xdvshow_render();

      if (input_unlock_func) {
        (*input_unlock_func)();
      }

      gettimeofday(&tv, NULL);
      printf("%d msec\n", (int)((tv.tv_sec - tv_prev.tv_sec)*1000000 +
                              (tv.tv_usec - tv_prev.tv_usec)) / 1000);
      memcpy(&tv_prev, &tv, sizeof(tv));
    }
  } else {
    while (1) {
      (*input_func)(&videodata);
      dv_decode_full_frame(dv_dec,
                           videodata,
                           param.decode_format,
                           param.pixels,
                           param.pitches);

      xdvshow_render();

      if (input_unlock_func) {
        (*input_unlock_func)();
      }

/*
      (*input_func)(videodata2);
      dv_decode_full_frame(dv_dec,
                           videodata2,
                           param.decode_format,
                           param.pixels,
                           param.pitches);

      xdvshow_render();
*/
    }
  }

  return(NULL);
}
