/*
 * Copyright (c) 1999, 2000, 2001, 2002 WIDE Project
 * All rights reserved.
 *
 * Author : Akimichi OGAWA (akimichi@sfc.wide.ad.jp)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code MUST retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form MUST reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    MUST display the following acknowledgement:
 *      This product includes software developed by Akimichi OGAWA.
 * 4. The name of the author MAY NOT be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <stdio.h>
#include <sys/types.h>
#include <libdv/dv_types.h>

#include <SDL/SDL.h>
#include <SDL/SDL_syswm.h>

#include "xdvshow-sdl.h"

SDL_Surface *sdl_screen;
SDL_Overlay *overlay;
SDL_Rect rect;

int width, height;

static void _dv_center_window __P((SDL_Surface *));

int
xdvshow_open_window(char *window_name, struct x_params *param)
{
  int ret;
  const SDL_VideoInfo *video_info;
  int video_bpp;

  switch (param->dv_format_type) {
    case e_dv_system_525_60:
      width = 720;
      height = 480;
      break;

    case e_dv_system_625_50:
      width = 720;
      height = 576;
      break;

    default:
      return(-1);
  }

  ret = SDL_Init(SDL_INIT_VIDEO);
  if (ret < 0) {
    return(-1);
  }

  video_info = SDL_GetVideoInfo();
  switch (video_info->vfmt->BitsPerPixel) {
    case 16:
    case 32:
      video_bpp = video_info->vfmt->BitsPerPixel;
      break;

    default:
      video_bpp = 16;
      break;
  }

  sdl_screen = SDL_SetVideoMode(width, height, video_bpp, SDL_HWSURFACE);

  SDL_WM_SetCaption(window_name, "name");

#define	DV_FOURCC_YUY2	0x32595559
  overlay = SDL_CreateYUVOverlay(width, height, DV_FOURCC_YUY2, sdl_screen);

  _dv_center_window(sdl_screen);

  rect.x = 0;
  rect.y = 0;
  rect.w = overlay->w;
  rect.h = overlay->h;
  param->pixels[0] = overlay->pixels[0];
  param->pixels[1] = overlay->pixels[1];
  param->pixels[2] = overlay->pixels[2];
  param->pitches[0] = overlay->pitches[0];
  param->pitches[1] = overlay->pitches[1];
  param->pitches[2] = overlay->pitches[2];

  param->decode_format = e_dv_color_yuv;

  return(1);
}

int
xdvshow_close_window(void)
{
  return(1);
}

int
xdvshow_render(void)
{
  SDL_UnlockYUVOverlay(overlay);
  SDL_DisplayYUVOverlay(overlay, &rect);
  SDL_LockYUVOverlay(overlay);

  return(1);
}

static void
_dv_center_window(SDL_Surface *screen)
{
  SDL_SysWMinfo info;

  SDL_VERSION(&info.version);
  if ( SDL_GetWMInfo(&info) > 0 ) {
    int x, y;
    int w, h;
    if (info.subsystem == SDL_SYSWM_X11) {
      info.info.x11.lock_func();
      w = DisplayWidth(info.info.x11.display,
                       DefaultScreen(info.info.x11.display));
      h = DisplayHeight(info.info.x11.display,
                        DefaultScreen(info.info.x11.display));
      x = (w - screen->w)/2;
      y = (h - screen->h)/2;
      XMoveWindow(info.info.x11.display, info.info.x11.wmwindow, x, y);
      info.info.x11.unlock_func();
    }
  }
}
