/*
 * Copyright (c) 1999, 2000, 2001, 2002 WIDE Project
 * All rights reserved.
 *
 * Author : Akimichi OGAWA (akimichi@sfc.wide.ad.jp)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code MUST retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form MUST reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    MUST display the following acknowledgement:
 *      This product includes software developed by Akimichi OGAWA.
 * 4. The name of the author MAY NOT be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/shm.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <signal.h>
#include <netdb.h>
#include <net/if.h>
#include <arpa/inet.h>

#include <libdv/dv_types.h>

#include <RTP.h>

#include <xdvshow-shm.h>
#include <xdvshow-flags.h>

#define		DVFRAME_READY		0x00000001
#define		DVFRAME_COPYING		0x00000002
#define		DVFRAME_DATA_READY	0x00000008

#define		DVFRAME_MAX_SIZE	144000

#define		SCT_HEADER	0x0
#define		SCT_SUBCODE	0x1
#define		SCT_VAUX	0x2
#define		SCT_AUDIO	0x3
#define		SCT_VIDEO	0x4

#define		SHM_FRAME_BUF_NUM	6

extern u_int32_t flags;
extern u_int32_t dv_format_type;

struct shm_frame {
  struct shm_frame *next;
  struct shm_frame_buf *frame_buf;
};

struct shm_frame_buf {
  u_long lock;
  u_char data[DVFRAME_MAX_SIZE];	/* size of PAL since PAL is larger */
};

struct __xdvshow_shm {
  int parent_pid;
  int child_pid;

  int soc;

  int shmid;
  int framebufs;

  u_long *shm_buf;

  struct shm_frame *shm_frame;
  struct shm_frame *audio_shm_frame;

  struct shm_frame_buf *locked_shm_framebuf;
  struct shm_frame_buf *locked_audio_shm_framebuf;
};

static struct __xdvshow_shm _xdvshow_shm;

static int _xdvshow_alloc_shm __P((void));
static int _xdvshow_attach_shm __P((void));
static int _xdvshow_prepare_socket __P((struct xdvshow_rtp_param *));
static int _xdvshow_prepare_multicast __P((struct xdvshow_rtp_param *));
static int _xdvshow_recv_dvrtp __P((void));

static int _multicast_str2addr __P((struct xdvshow_rtp_param *));
static int _chk_multicast_addr __P((struct xdvshow_rtp_param *));
static int _chk_multicast_family __P((struct xdvshow_rtp_param *));
static int _multicast_join __P((struct xdvshow_rtp_param *));

void
xdvshow_signal_shm(int sig)
{
  if (getpid() == _xdvshow_shm.parent_pid) {
    kill(_xdvshow_shm.child_pid, SIGINT);
  }
  else {
    kill(_xdvshow_shm.parent_pid, SIGINT);
  }

  shmdt(_xdvshow_shm.shm_buf);
  shmctl(_xdvshow_shm.shmid, IPC_RMID, NULL);

  exit(0);
}

int
xdvshow_prepare_shm(void *data)
{
  int ret;
  int pid;

  struct xdvshow_rtp_param *rtp_param = data;

  if (rtp_param == NULL) {
    return(1);
  }

  ret = _xdvshow_alloc_shm();
  if (ret < 0) {
    return(1);
  }

  _xdvshow_shm.parent_pid = getpid();
  if ((pid = fork()) == 0) {
    ret = _xdvshow_prepare_socket(rtp_param);
    if (ret < 0) {
      kill(_xdvshow_shm.parent_pid, SIGINT);
      exit(1);
    }

    if (rtp_param->multi_addr_str != NULL) {
      ret = _xdvshow_prepare_multicast(rtp_param);
      if (ret < 0) { 
        kill(_xdvshow_shm.parent_pid, SIGINT);
        exit(1);
      }
    }

    ret = _xdvshow_attach_shm();
    if (ret < 0) {
      kill(_xdvshow_shm.parent_pid, SIGINT);
      exit(1);
    }

    /* THIS IS NOT SUPPOSED TO COME BACK */
    _xdvshow_recv_dvrtp();

    exit(0);
  }
  else {
    _xdvshow_shm.child_pid = pid;

    ret = _xdvshow_attach_shm();
    if (ret < 0) {
      return(1);
      kill(_xdvshow_shm.child_pid, SIGINT);
    }

    _xdvshow_shm.locked_shm_framebuf = _xdvshow_shm.shm_frame->frame_buf;
    _xdvshow_shm.locked_audio_shm_framebuf = _xdvshow_shm.audio_shm_frame->frame_buf;
  }

  return(1);
}

void
xdvshow_close_shm()
{
  shmdt(_xdvshow_shm.shm_buf);
}

int
xdvshow_read_unlock()
{
  _xdvshow_shm.locked_shm_framebuf->lock = DVFRAME_READY;

  return(1);
}

int
xdvshow_read_shm(u_char **dvframe)
{
  while (1) {
    if (_xdvshow_shm.shm_frame->next->frame_buf->lock == DVFRAME_DATA_READY) {
      _xdvshow_shm.shm_frame = _xdvshow_shm.shm_frame->next;
      _xdvshow_shm.shm_frame->frame_buf->lock = DVFRAME_COPYING;
      break;
    }
  }

   *dvframe = _xdvshow_shm.shm_frame->frame_buf->data;
  _xdvshow_shm.locked_shm_framebuf = _xdvshow_shm.shm_frame->frame_buf;

  return(1);
}

int
xdvshow_read_audio_unlock()
{
  _xdvshow_shm.locked_audio_shm_framebuf->lock = DVFRAME_READY;

  return(1);
}

int
xdvshow_read_audio_shm(u_char **audioframe)
{
  while (1) {
    if (_xdvshow_shm.audio_shm_frame->next->frame_buf->lock == DVFRAME_DATA_READY) {
      _xdvshow_shm.audio_shm_frame = _xdvshow_shm.audio_shm_frame->next;
      _xdvshow_shm.audio_shm_frame->frame_buf->lock = DVFRAME_COPYING;
      break;
    }
  }

  *audioframe = _xdvshow_shm.audio_shm_frame->frame_buf->data;
  _xdvshow_shm.locked_audio_shm_framebuf = _xdvshow_shm.audio_shm_frame->frame_buf;

  return(1);
}

static int
_xdvshow_alloc_shm()
{
  int shmid;
  int framebufs = SHM_FRAME_BUF_NUM;

  shmid = shmget(IPC_PRIVATE,
                 framebufs*sizeof(struct shm_frame_buf)*2,	/* audio and video */
                 0600);
  if (shmid < 0) {
    perror("shmget");
    return(-1);
  }

  _xdvshow_shm.shmid = shmid;
  _xdvshow_shm.framebufs = framebufs;

  return(1);
}

static int
_xdvshow_attach_shm()
{
  u_long *shm_buf;

  int i;
  struct shm_frame *shm_frame = NULL;
  struct shm_frame *shm_frame_last = NULL;

  if ((shm_buf = shmat(_xdvshow_shm.shmid, 0, 0)) == (void *)-1) {
    perror("shmat");
    return(-1);
  }

  _xdvshow_shm.shm_buf = shm_buf;

  for (i=0; i<_xdvshow_shm.framebufs; i++) {
    shm_frame = (struct shm_frame *)malloc(sizeof(struct shm_frame));
    memset(shm_frame, 0, sizeof(struct shm_frame));

    shm_frame->frame_buf = (struct shm_frame_buf *)(shm_buf + (i * sizeof(struct shm_frame_buf))/4);
    shm_frame->frame_buf->lock = DVFRAME_READY;

    if (shm_frame_last == NULL) {
      _xdvshow_shm.shm_frame = shm_frame;
    }
    else {
      shm_frame_last->next = shm_frame;
    }
    shm_frame_last = shm_frame;
  }
  shm_frame->next = _xdvshow_shm.shm_frame;

  shm_frame = NULL;
  shm_frame_last = NULL;
  for (i=0; i<_xdvshow_shm.framebufs; i++) {
    shm_frame = (struct shm_frame *)malloc(sizeof(struct shm_frame));
    memset(shm_frame, 0, sizeof(struct shm_frame));

    shm_frame->frame_buf = (struct shm_frame_buf *)(shm_buf + ((_xdvshow_shm.framebufs + i) * sizeof(struct shm_frame_buf))/4);
    shm_frame->frame_buf->lock = DVFRAME_READY;

    if (shm_frame_last == NULL) {
      _xdvshow_shm.audio_shm_frame = shm_frame;
    }
    else {
      shm_frame_last->next = shm_frame;
    }
    shm_frame_last = shm_frame;
  }
  shm_frame->next = _xdvshow_shm.audio_shm_frame;

  return(1);
}

static int
_xdvshow_prepare_socket(struct xdvshow_rtp_param *rtp_param)
{
  int soc;

  struct sockaddr_in *s_in = (struct sockaddr_in *)&rtp_param->s_addr;
#ifdef	ENABLE_INET6
  struct sockaddr_in6 *s_in6 = (struct sockaddr_in6 *)&rtp_param->s_addr;
#endif	/* ENABLE_INET6 */

#ifdef	ENABLE_INET6
  if (rtp_param->s_addr.ss_family == AF_INET) {
#ifndef	NO_SS_LEN
    s_in->sin_len = sizeof(struct sockaddr_in);
#endif	/* NO_SS_LEN */
    s_in->sin_port = htons(rtp_param->port);
    s_in->sin_addr.s_addr = htonl(INADDR_ANY);
  } else if (rtp_param->s_addr.ss_family == AF_INET6) {
#ifndef	NO_SS_LEN
    s_in6->sin6_len = sizeof(struct sockaddr_in6);
#endif	/* NO_SS_LEN */
    s_in6->sin6_port = htons(rtp_param->port);
  } else {
    printf("Unknown family : %d\n", rtp_param->s_addr.ss_family);
    return(-1);
  }
#else	/* ENABLE_INET6 */
#ifndef	NO_SS_LEN
  s_in->sin_len = sizeof(struct sockaddr_in);
#endif	/* NO_SS_LEN */
  s_in->sin_port = htons(rtp_param->port);
  s_in->sin_addr.s_addr = htonl(INADDR_ANY);
#endif	/* ENABLE INET6 */

  soc = socket(rtp_param->s_addr.ss_family, SOCK_DGRAM, 0);
  if (soc < 0) {
    perror("socket");
    return(-1);
  }

#ifndef	NO_SS_LEN
  if (bind(soc, (struct sockaddr *)&rtp_param->s_addr, rtp_param->s_addr.ss_len) < 0) {
#else	/* NO_SS_LEN */
  if (bind(soc, (struct sockaddr *)&rtp_param->s_addr, sizeof(rtp_param->s_addr)) < 0) {
#endif	/* NO_SS_LEN */
    perror("bind");
    return(-1);
  }

  _xdvshow_shm.soc = soc;

  return(1);
}

static int
_multicast_str2addr(struct xdvshow_rtp_param *rtp_param)
{
#ifdef	ENABLE_INET6
  struct addrinfo hints, *res;

  memset(&hints, 0, sizeof(hints));

  hints.ai_socktype = SOCK_DGRAM;
  hints.ai_protocol = IPPROTO_UDP;

  if (getaddrinfo(rtp_param->multi_addr_str, NULL, &hints, &res) != 0) {
    perror("getaddrinfo");
    return(-1);
  }

  memcpy(&rtp_param->multi_s_addr, res->ai_addr, res->ai_addrlen);
#else	/* ENABLE_INET6 */
  struct sockaddr_in *s_in = (struct sockaddr_in *)&rtp_param->multi_s_addr;

  s_in->sin_addr.s_addr = inet_addr(rtp_param->multi_addr_str);
  if (s_in->sin_addr.s_addr == -1) {
    return(-1);
  }
#endif	/* ENABLE_INET6 */

  return(1);
}

static int
_chk_multicast_addr(struct xdvshow_rtp_param *rtp_param)
{
  struct sockaddr_in *s_in = (struct sockaddr_in *)&rtp_param->multi_s_addr;
#ifdef	ENABLE_INET6
  struct sockaddr_in6 *s6_in = (struct sockaddr_in6 *)&rtp_param->multi_s_addr;
#endif	/* ENABLE_INET6 */

  switch (rtp_param->multi_s_addr.ss_family) {
    case AF_INET:
      if (!IN_MULTICAST(ntohl(s_in->sin_addr.s_addr))) {
        printf("ERROR : [%s] is not multicast address\n",
               rtp_param->multi_addr_str);
        return(-1);
      }
      break;

#ifdef	ENABLE_INET6
    case AF_INET6:
      if (!IN6_IS_ADDR_MULTICAST(&s6_in->sin6_addr)) {
        printf("ERROR : [%s] is not multicast address\n",
               rtp_param->multi_addr_str);
        return(-1);
      }
      break;
#endif	/* ENABLE_INET6 */

    default:
      printf("ERROR : unknown multicast addr family : %d\n",
             rtp_param->multi_s_addr.ss_family);
      return(-1);
      break;
  }

  return(1);
}

static int
_multicast_join(struct xdvshow_rtp_param *rtp_param)
{
  struct ip_mreq mreq;
  struct sockaddr_in *s_in = (struct sockaddr_in *)&rtp_param->multi_s_addr;
  struct sockaddr_in *base_s_in = (struct sockaddr_in *)&rtp_param->s_addr;
#ifdef	ENABLE_INET6
  struct ipv6_mreq mreq6;
  struct sockaddr_in6 *s6_in = (struct sockaddr_in6 *)&rtp_param->multi_s_addr;

  memset(&mreq6, 0, sizeof(mreq6));
#endif	/* ENABLE_INET6 */

  memset(&mreq, 0, sizeof(mreq));

  switch (rtp_param->multi_s_addr.ss_family) {
    case AF_INET:
      mreq.imr_multiaddr.s_addr = s_in->sin_addr.s_addr;
      mreq.imr_interface.s_addr = base_s_in->sin_addr.s_addr;

      if (setsockopt(_xdvshow_shm.soc, IPPROTO_IP, IP_ADD_MEMBERSHIP,
                     (char *)&mreq, sizeof(mreq)) < 0) {
        perror("ip_add_membership");
        printf("multi addr     : %s\n", inet_ntoa(mreq.imr_multiaddr));
        printf("interface addr : %s\n", inet_ntoa(mreq.imr_interface));
      }
      break;

#ifdef	ENABLE_INET6
    case AF_INET6:
      memcpy((char *)&mreq6.ipv6mr_multiaddr,
             (char *)&s6_in->sin6_addr,
             sizeof(s6_in->sin6_addr));

      if (rtp_param->multi_ifname == NULL) {
        printf("ERROR : multicast interface required\n");
        printf("  ex) xdvshow -j ff02::ab:cd -M fxp0\n");
        return(-1);
      }
      mreq6.ipv6mr_interface = if_nametoindex(rtp_param->multi_ifname);

#ifdef  IPV6_ADD_MEMBERSHIP
      if (setsockopt(_xdvshow_shm.soc, IPPROTO_IPV6, IPV6_ADD_MEMBERSHIP,
                     (char *)&mreq6, sizeof(mreq6)) < 0) {
#else   /* IPV6_ADD_MEMBERSHIP */
      if (setsockopt(_xdvshow_shm.soc, IPPROTO_IPV6, IPV6_JOIN_GROUP,
                     (char *)&mreq6, sizeof(mreq6)) < 0) {
#endif  /* IPV6_ADD_MEMBERSHIP */
        perror("ip_add_membership");
      }
      break;
#endif	/* ENABLE_INET6 */

    default:
      return(-1);
      break;
  }

  return(1);
}


static int
_chk_multicast_family(struct xdvshow_rtp_param *rtp_param)
{
  if (rtp_param->s_addr.ss_family !=
        rtp_param->multi_s_addr.ss_family) {
    printf("Protocol family of DV/RTP socket and multicast join address mismatch\n");

    switch (rtp_param->s_addr.ss_family) {
      case AF_INET:
        printf("DV/RTP socket family   : IPv4 (AF_INET)\n");
        break;

#ifdef ENABLE_INET6
      case AF_INET6:
        printf("DV/RTP socket family   : IPv6 (AF_INET6)\n");
        break;
#endif  /* ENABLE_INET6 */

      default:
        break;
    }
    switch (rtp_param->multi_s_addr.ss_family) {
      case AF_INET:
        printf("multicast join address : IPv4 (AF_INET)\n");
        break;

#ifdef ENABLE_INET6
      case AF_INET6:
        printf("multicast join address : IPv6 (AF_INET6)\n");
        break;
#endif  /* ENABLE_INET6 */

      default:
        break;
    }

    return(-1);
  }

  return(1);
}

static int
_xdvshow_prepare_multicast(struct xdvshow_rtp_param *rtp_param)
{
  int ret;

  ret = _multicast_str2addr(rtp_param);
  if (ret < 0) {
    return(-1);
  }

  ret = _chk_multicast_addr(rtp_param);
  if (ret < 0) {
    return(-1);
  }

  ret = _chk_multicast_family(rtp_param);
  if (ret < 0) {
    return(-1);
  }

  ret = _multicast_join(rtp_param);
  if (ret < 0) {
    return(-1);
  }

  return(1);
}

static int
_xdvshow_recv_dvrtp()
{
  int n;
  int i;
  struct sockaddr_storage from;
  int length = sizeof(from);
#define		MAX_PKT_LEN	2000
  u_long recvbuf[MAX_PKT_LEN];
  u_long *recvbuf_ptr;

  u_char *dvdata;
  u_char *audio_dvdata;

  int dbn, dseq, sct;

  int dseq_max;

  switch (dv_format_type) {
    case e_dv_system_525_60:
      dseq_max = 9;
      break;

    case e_dv_system_625_50:
      dseq_max = 11;
      break;

    default:
      return(-1);
  }

  dvdata = _xdvshow_shm.shm_frame->frame_buf->data;
  audio_dvdata = _xdvshow_shm.audio_shm_frame->frame_buf->data;

  while (1) {
    n = recvfrom(_xdvshow_shm.soc, (char *)&recvbuf, sizeof(recvbuf), 0,
                 (struct sockaddr *)&from, &length);
    if (n < 1) {
      return(-1);
    }

    for (i=0; i<(n - sizeof(rtp_hdr_t))/80; i++) {
      recvbuf_ptr = &recvbuf[(sizeof(rtp_hdr_t)+80*i)/4];
      dbn  = (ntohl(*recvbuf_ptr) >> 8) & 0xff;
      dseq = (ntohl(*recvbuf_ptr) >> 20) & 0xf;
      sct  = (ntohl(*recvbuf_ptr) >> 29) & 0x7;

/*
      if (sct > 4) {
        printf("INVALID sct : %d\n", sct);
        continue;
      }
*/

      switch (sct) {
/*
        case SCT_HEADER:
          memcpy(&dvdata[(dseq * 150) * 80],
                 recvbuf_ptr,
                 80);
          break;

        case SCT_SUBCODE:
          memcpy(&dvdata[(dseq * 150 + dbn + 1) * 80],
                 recvbuf_ptr,
                 80);
          break;
*/

        case SCT_VAUX:
          memcpy(&dvdata[(dseq * 150 + dbn + 3) * 80],
                 recvbuf_ptr,
                 80);
          break;

        case SCT_AUDIO:
          memcpy(&dvdata[(dseq * 150 + 6 + dbn*16) * 80],
                 recvbuf_ptr,
                 80);
          if (flags & USE_AUDIO) {
            memcpy(&audio_dvdata[(dseq * 150 + 6 + dbn*16) * 80],
                   recvbuf_ptr,
                   80);
            if (dseq == dseq_max && dbn == 8) {
              _xdvshow_shm.audio_shm_frame->frame_buf->lock = DVFRAME_DATA_READY;
              while (1) {
                _xdvshow_shm.audio_shm_frame = _xdvshow_shm.audio_shm_frame->next;
                if (_xdvshow_shm.audio_shm_frame->frame_buf->lock != DVFRAME_COPYING) {
                  break;
                }
              }
              audio_dvdata = _xdvshow_shm.audio_shm_frame->frame_buf->data;
            }
          }
          break;

        case SCT_VIDEO:
          memcpy(&dvdata[(dseq * 150 + 7 + dbn + dbn/15) * 80],
                 recvbuf_ptr,
                 80);
          if (dseq == dseq_max && dbn == 134) {
            _xdvshow_shm.shm_frame->frame_buf->lock = DVFRAME_DATA_READY;

            while (1) {
              _xdvshow_shm.shm_frame = _xdvshow_shm.shm_frame->next;
              if (_xdvshow_shm.shm_frame->frame_buf->lock != DVFRAME_COPYING) {
                break;
              }
            }
            dvdata = _xdvshow_shm.shm_frame->frame_buf->data;
          }
          break;

        default:
          break;
      }
    }
  }

  return(1);
}
