/*
 * Copyright (c) 1999, 2000, 2001, 2002 WIDE Project
 * All rights reserved.
 *
 * Author : Akimichi OGAWA (akimichi@sfc.wide.ad.jp)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code MUST retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form MUST reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    MUST display the following acknowledgement:
 *      This product includes software developed by Akimichi OGAWA.
 * 4. The name of the author MAY NOT be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

#include <libdv/dv.h>

#include "xdvshow-x11.h"
#include "xdvshow-flags.h"

#define	USE_COLOR	0x00000001
#define USE_GREY	0x00000002

#define  WIDTH		720

#define  PAL_HEIGHT		576
#define  NTSC_HEIGHT		480

extern int flags;

static u_long  ***color_table;
static u_long color_depth = 4;

static Display *display;
static int screen;
static Window root, window;
static XImage *ximage;
static GC gc;
static int width, height;
static int real_width, real_height;

static int _init_color_table __P((int));

struct x_params *x_param;

static int
_init_color_table(int type)
{
  int r, g, b;
  XColor color;
  u_long tmp_depth = (1 << color_depth);

  memset(&color, 0, sizeof(color));

  color_table = (u_long ***)malloc(sizeof(u_long ***) * tmp_depth);
  if (color_table == NULL) {
    perror("malloc");
    return(-1);
  }

  for (r=0; r<tmp_depth; r++) {
    color_table[r] = (u_long **)malloc(sizeof(u_long **) * tmp_depth);
    if (color_table[r] == NULL) {
      perror("malloc");
      return(-1);
    }

    for (g=0; g<tmp_depth; g++) {
      color_table[r][g] = (u_long *)malloc(sizeof(u_long *) * tmp_depth);
      if (color_table[r][g] == NULL) {
        perror("malloc");
        return(-1);
      }

      for (b=0; b<tmp_depth; b++) {
        color.red   = 65535 / tmp_depth * r;
        color.green = 65535 / tmp_depth * g;
        color.blue  = 65535 / tmp_depth * b;
        if (!XAllocColor(display, DefaultColormap(display, screen), &color)) {
          printf("XAllocColor failed, r=%d, g=%d, b=%d\n", r, g, b);
          return(-1);
        }
        color_table[r][g][b] = color.pixel;
      }
    }
  }

  return(1);
}

int
xdvshow_open_window(char *window_name, struct x_params *param)
{
/*
  XEvent xevent;
*/

  x_param = param;

  param->pixels[0] = malloc(720 * 576 * 3);
  memset(param->pixels[0], 0, 720 * 576 * 3);
  param->pixels[1] = NULL;
  param->pixels[2] = NULL;

  param->pitches[0] = 720 * 3;
  param->pitches[1] = 0;
  param->pitches[2] = 0;

  param->decode_format = e_dv_color_rgb;

  switch (param->dv_format_type) {
    case e_dv_system_525_60:
      width = WIDTH;
      height = NTSC_HEIGHT;
      real_width = WIDTH;
      real_height = NTSC_HEIGHT;
      break;

    case e_dv_system_625_50:
      width = WIDTH;
      height = PAL_HEIGHT;
      real_width = WIDTH;
      real_height = PAL_HEIGHT;
      break;

    default:
      return(-1);
  }

  if (flags & HALF_SIZE) {
    width /= 2;
    height /= 2;
  }

  display = XOpenDisplay(NULL);
  if (display == NULL) {
    perror("XOpenDisplay");
    return(-1);
  }

  screen = DefaultScreen(display);
  root = RootWindow(display, screen);

  _init_color_table(0);

  window = XCreateSimpleWindow(display, root, 0, 0, width, height,
                               2, color_table[0][0][0],
                               color_table[0][0][0]);
  gc = XCreateGC(display, window, 0, 0);

  XMapWindow(display, window);

  XStoreName(display, window, window_name);
  XFlush(display);

/*
  XSelectInput(display, window, ExposureMask);
  XWindowEvent(display, window, ExposureMask, &xevent);
*/

  ximage = XCreateImage(display, DefaultVisual(display, screen),
                        DefaultDepth(display, screen),
                        ZPixmap, 0, 0, width, height, 32, 0);
  if (ximage == NULL) {
    printf("XCreateImage() failed\n");
    return(-1);
  }

  ximage->data = (unsigned char *)malloc(ximage->bytes_per_line*height);
  if (ximage->data == NULL) {
    perror("XCreateImage data failed\n");
    return(-1);
  }

  XFlush(display);

  return(1);
}

int
xdvshow_close_window(void)
{
  return(1);
}

int
xdvshow_render(void)
{
  u_char r, g, b;
  int16_t x, y;
  register u_char *ptr;

  u_long color_shift;

  ptr = &x_param->pixels[0][0];

  color_shift = (8 - color_depth);

  if (flags & HALF_SIZE) {
    for (y=0; y<real_height; y++) {
      if (y % 2) {
        ptr += real_width * 3;
        continue;
      }

      for (x=0; x<real_width; x++) {
        if (x % 2) {
          ptr += 3;
          continue;
        }

        r = *ptr;
        r >>= color_shift;
        ptr++;

        g = *ptr;
        g >>= color_shift;
        ptr++;

        b = *ptr;
        b >>= color_shift;
        ptr++;

        XPutPixel(ximage, x/2, y/2, color_table[r][g][b]);
      }
    }
  } else {
    for (y=0; y<height; y++) {
      for (x=0; x<width; x++) {
        r = *ptr;
        r >>= color_shift;
        ptr++;

        g = *ptr;
        g >>= color_shift;
        ptr++;

        b = *ptr;
        b >>= color_shift;
        ptr++;

        XPutPixel(ximage, x, y, color_table[r][g][b]);
      }
    }
  }

  XPutImage(display, window, gc, ximage, 0, 0, 0, 0, width, height);
  XFlush(display);

  return(1);
}
