/*
 * Copyright (c) 1999, 2000, 2001, 2002 WIDE Project
 * All rights reserved.
 *
 * Author : Akimichi OGAWA (akimichi@sfc.wide.ad.jp)
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code MUST retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form MUST reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    MUST display the following acknowledgement:
 *      This product includes software developed by Akimichi OGAWA.
 * 4. The name of the author MAY NOT be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <stdio.h>
#include <sys/types.h>
#include <libdv/dv_types.h>

#include <sys/ipc.h>
#include <sys/shm.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/Xvlib.h>
#include <X11/extensions/XShm.h>

#include "xdvshow-xv.h"

extern int flags;

int width, height;

static Display *display;
static int screen;
static Window root, window;
static XvAdaptorInfo *ad_info;
static int ad_cnt;
static int xv_port;
static GC gc;
static XShmSegmentInfo shminfo;
static XvImage *xv_image;
static int xv_format;

int
xdvshow_open_window(char *window_name, struct x_params *param)
{
  int ret;
  XTextProperty x_wname;
  XSizeHints hints;
  XWMHints wmhints;
  XGCValues values;

  switch (param->dv_format_type) {
    case e_dv_system_525_60:
      width = 720;
      height = 480;
      break;

    case e_dv_system_625_50:
      width = 720;
      height = 576;
      break;

    default:
      return(-1);
  }

  display = XOpenDisplay(NULL);
  if (display == NULL) {
    return(-1);
  }

  screen = DefaultScreen(display);

  ret = XvQueryAdaptors(display, root, &ad_cnt, &ad_info);
  if (ret != 1) {
    return(-1);
  }

  XvGrabPort(display, xv_port, CurrentTime); 

  hints.flags = PSize | PMaxSize | PMinSize;
  hints.min_width = width / 16;
  hints.min_height = height / 16;

  hints.max_width = 2048;
  hints.max_height = 2048;

  wmhints.input = True;
  wmhints.flags = InputHint;

  window = XCreateSimpleWindow(display,
                               root,
                               0, 0,
                               width, height,
                               0,
                               XWhitePixel(display, screen),
                               XBlackPixel(display, screen));

  XStringListToTextProperty(&window_name, 1, &x_wname);
  XSetWMProperties(display, window,
                   &x_wname, &x_wname,
                   NULL, 0,
                   &hints, &wmhints, NULL);

  gc = XCreateGC(display, window, 0, &values);

  xv_image = XvShmCreateImage(display,
                              xv_port, xv_format,
                              param->pixels[0],
                              width, height,
                              &shminfo);

  return(1);
}

int
xdvshow_close_window(void)
{
  return(1);
}

int
xdvshow_render(void)
{
  return(1);
}
